@@ -20,21 +20,30 @@ module Stomp
     end
 
     def parse_stomp_url(login)
+      original_verbose, $VERBOSE = $VERBOSE, nil # shut off warnings
       regexp = /^stomp:\/\/#{URL_REPAT}/
-      return false unless url = regexp.match(login)
-
+      url = regexp.match(login)
+      $VERBOSE = original_verbose
+      return false unless url
+      @login =  url[3] || ""
+      @passcode = url[4] || ""
+      @host = url[5]
+      @port = url[6].to_i
       @parameters = { :reliable => false,
-                      :hosts => [ { :login => url[3] || "",
-                                    :passcode => url[4] || "",
-                                    :host => url[5],
-                                    :port => url[6].to_i} ] }
+                      :hosts => [ { :login => @login,
+                                    :passcode => @passcode,
+                                    :host => @host,
+                                    :port => @port} ] }
       true
     end
 
     # e.g. failover://(stomp://login1:passcode1@localhost:61616,stomp://login2:passcode2@remotehost:61617)?option1=param
     def parse_failover_url(login)
       rval = nil
-      if md = FAILOVER_REGEX.match(login)
+      original_verbose, $VERBOSE = $VERBOSE, nil # shut off warnings
+      md = FAILOVER_REGEX.match(login)
+      $VERBOSE = original_verbose
+      if md
         finhosts = parse_hosts(login)
 
         options = {}
@@ -66,15 +75,20 @@ module Stomp
     # setting an id in the SUBSCRIPTION header is described in the stomp protocol docs:
     # http://stomp.github.com/
     def set_subscription_id_if_missing(destination, headers)
-      headers[:id] = headers[:id] ? headers[:id] : headers['id']
-      if headers[:id] == nil
-        headers[:id] = Digest::SHA1.hexdigest(destination)
-      end
+      headers[:id] = build_subscription_id(destination, headers)
+    end
+
+    def build_subscription_id(destination, headers)
+      return headers[:id] until headers[:id].nil?
+      return headers['id'] until headers['id'].nil?
+      # p [ "DBBSID1", destination, headers ]
+      Digest::SHA1.hexdigest(destination)
     end
 
     # Parse a stomp URL.
     def parse_hosts(url)
       hosts = []
+      original_verbose, $VERBOSE = $VERBOSE, nil # shut off warnings
       host_match = /stomp(\+ssl)?:\/\/#{URL_REPAT}/
       url.scan(host_match).each do |match|
         host = {}
@@ -85,6 +99,7 @@ module Stomp
         host[:port] = match[6].to_i
         hosts << host
       end
+      $VERBOSE = original_verbose
       hosts
     end
 
@@ -95,7 +110,7 @@ module Stomp
       @parameters[:hosts].each do |hv|
         # Validate port requested
         raise ArgumentError.new("empty :port value in #{hv.inspect}") if hv[:port] == ''
-        unless hv[:port].nil? 
+        unless hv[:port].nil?
           tpv = hv[:port].to_i
           raise ArgumentError.new("invalid :port value=#{tpv} from #{hv.inspect}") if tpv < 1 || tpv > 65535
         end
@@ -107,8 +122,7 @@ module Stomp
       if @parameters[:reliable] && @start_timeout > 0
         warn "WARN detected :reliable == true and :start_timeout > 0"
         warn "WARN this may cause incorrect fail-over behavior"
-        warn "WARN use :start_timeout => 0 to correct"
-        warn "WARN !! :start_timeout default will change to 0 in the next release"
+        warn "WARN use :start_timeout => 0 to correct fail-over behavior"
       end
     end
 
@@ -149,7 +163,10 @@ module Stomp
 
     def find_receipt_listener(message)
       listener = @receipt_listeners[message.headers['receipt-id']]
-      listener.call(message) if listener
+      if listener
+         listener.call(message)
+         @receipt_listeners.delete(message.headers['receipt-id'])
+      end
     end
 
     def create_listener_maps
@@ -191,4 +208,3 @@ module Stomp
   end # class Client
 
 end # module Stomp
-
