@@ -46,6 +46,18 @@ module Stomp
       ssl ? 61612 : 61613
     end
 
+    # SSL Helper
+    def self.ssl_v2xoptions()
+        require 'openssl' unless defined?(OpenSSL)
+        # Mimic code in later versions of Ruby 2.x (and backported to later
+        # versions of 1.9.3).
+        opts = OpenSSL::SSL::OP_ALL
+        opts &= ~OpenSSL::SSL::OP_DONT_INSERT_EMPTY_FRAGMENTS if defined?(OpenSSL::SSL::OP_DONT_INSERT_EMPTY_FRAGMENTS)
+        opts |= OpenSSL::SSL::OP_NO_COMPRESSION if defined?(OpenSSL::SSL::OP_NO_COMPRESSION)
+        opts |= OpenSSL::SSL::OP_NO_SSLv2 if defined?(OpenSSL::SSL::OP_NO_SSLv2)
+        opts |= OpenSSL::SSL::OP_NO_SSLv3 if defined?(OpenSSL::SSL::OP_NO_SSLv3)
+    end
+
     # A new Connection object can be initialized using two forms:
     #
     # Hash (this is the recommended Connection initialization method):
@@ -55,28 +67,33 @@ module Stomp
     #       {:login => "login1", :passcode => "passcode1", :host => "localhost", :port => 61616, :ssl => false},
     #       {:login => "login2", :passcode => "passcode2", :host => "remotehost", :port => 61617, :ssl => false}
     #     ],
-    #     :reliable => true,
-    #     :initial_reconnect_delay => 0.01,
-    #     :max_reconnect_delay => 30.0,
-    #     :use_exponential_back_off => true,
-    #     :back_off_multiplier => 2,
-    #     :max_reconnect_attempts => 0,
-    #     :randomize => false,
-    #     :connect_timeout => 0,
-    #     :connect_headers => {},
-    #     :parse_timeout => 5,
-    #     :logger => nil,
-    #     :dmh => false,
-    #     :closed_check => true,
-    #     :hbser => false,
-    #     :stompconn => false,
-    #     :usecrlf => false,
-    #     :max_hbread_fails => 0,
-    #     :max_hbrlck_fails => 0,
-    #     :fast_hbs_adjust => 0.0,
-    #     :connread_timeout => 0,
-    #     :tcp_nodelay => true,
-    #     :start_timeout => 10,
+    #     # These are the default parameters and do not need to be set
+    #     :reliable => true,                  # reliable (use failover)
+    #     :initial_reconnect_delay => 0.01,   # initial delay before reconnect (secs)
+    #     :max_reconnect_delay => 30.0,       # max delay before reconnect
+    #     :use_exponential_back_off => true,  # increase delay between reconnect attpempts
+    #     :back_off_multiplier => 2,          # next delay multiplier
+    #     :max_reconnect_attempts => 0,       # retry forever, use # for maximum attempts
+    #     :randomize => false,                # do not radomize hosts hash before reconnect
+    #     :connect_timeout => 0,              # Timeout for TCP/TLS connects, use # for max seconds
+    #     :connect_headers => {},             # user supplied CONNECT headers (req'd for Stomp 1.1+)
+    #     :parse_timeout => 5,                # IO::select wait time on socket reads
+    #     :logger => nil,                     # user suplied callback logger instance
+    #     :dmh => false,                      # do not support multihomed IPV4 / IPV6 hosts during failover
+    #     :closed_check => true,              # check first if closed in each protocol method
+    #     :hbser => false,                    # raise on heartbeat send exception
+    #     :stompconn => false,                # Use STOMP instead of CONNECT
+    #     :usecrlf => false,                  # Use CRLF command and header line ends (1.2+)
+    #     :max_hbread_fails => 0,             # Max HB read fails before retry.  0 => never retry
+    #     :max_hbrlck_fails => 0,             # Max HB read lock obtain fails before retry.  0 => never retry
+    #     :fast_hbs_adjust => 0.0,            # Fast heartbeat senders sleep adjustment, seconds, needed ...
+    #                                         # For fast heartbeat senders.  'fast' == YMMV.  If not
+    #                                         # correct for your environment, expect unnecessary fail overs
+    #     :connread_timeout => 0,             # Timeout during CONNECT for read of CONNECTED/ERROR, secs
+    #     :tcp_nodelay => true,               # Turns on the TCP_NODELAY socket option; disables Nagle's algorithm
+    #     :start_timeout => 0,                # Timeout around Stomp::Client initialization
+    #     :sslctx_newparm => nil,             # Param for SSLContext.new
+    #     :ssl_post_conn_check => true,       # Further verify broker identity
     #   }
     #
     #   e.g. c = Stomp::Connection.new(hash)
@@ -98,6 +115,8 @@ module Stomp
       @hb_sent = true           # Assumed at first
       @hbs = @hbr = false       # Sending/Receiving heartbeats. Assume no for now.
       @jruby = false            # Assumed at first
+      # Initialize some variables
+      @closed, @socket, @hhas10, @rt, @st = true, nil, false, nil, nil
       if defined?(RUBY_ENGINE) && RUBY_ENGINE =~ /jruby/
         @jruby = true
       end
@@ -126,7 +145,9 @@ module Stomp
         @fast_hbs_adjust = 0.0 # Fast heartbeat senders sleep adjustment
         @connread_timeout = 0 # Connect read CONNECTED/ERROR timeout
         @tcp_nodelay = true # Disable Nagle
-        @start_timeout = 10 # Client only, startup timeout
+        @start_timeout = 0 # Client only, startup timeout
+        @sslctx_newparm = nil # SSLContext.new paramater
+        @ssl_post_conn_check = true # Additional broker verification
         warn "login looks like a URL, do you have the correct parameters?" if @login =~ /:\/\//
       end
 
@@ -135,6 +156,7 @@ module Stomp
       @transmit_semaphore = Mutex.new
       @read_semaphore = Mutex.new
       @socket_semaphore = Mutex.new
+      @gets_semaphore = Mutex.new
 
       @subscriptions = {}
       @failure = nil
@@ -163,6 +185,8 @@ module Stomp
       @max_hbrlck_fails = @parameters[:max_hbrlck_fails]
       @fast_hbs_adjust = @parameters[:fast_hbs_adjust]
       @connread_timeout = @parameters[:connread_timeout]
+      @sslctx_newparm = @parameters[:sslctx_newparm]
+      @ssl_post_conn_check = @parameters[:ssl_post_conn_check]
       #
       # Try to support Ruby 1.9.x and 2.x ssl.
       unless defined?(RSpec)
@@ -202,32 +226,51 @@ module Stomp
     end
 
     # Acknowledge a message, used when a subscription has specified
-    # client acknowledgement i.e. connection.subscribe("/queue/a", :ack => 'client').
-    # Accepts an optional transaction header ( :transaction => 'some_transaction_id' )
-    # Behavior is protocol level dependent, see the specifications or comments below.
-    def ack(message_id, headers = {})
+    # client acknowledgement e.g.:
+    #
+    #     connection.subscribe("/queue/a", :ack => 'client')
+    #
+    #     connection.subscribe("/queue/a", :ack => 'client-individual')
+    #
+    # as appropriate for the protocol level.
+    #
+    # Accepts an optional transaction header ( :transaction => 'some_transaction_id' ).
+    #
+    # When the connection protocol level is 1.0 or 1.1  the message_or_ack_id parameter
+    # should match the message-id header of the MESSAGE being acknowledged e.g.:
+    #
+    #     connection.ack(message.headers['message-id'])
+    #
+    # When the connection protocol level is 1.2 the message_or_ack_id parameter
+    # should match the ack header of the MESSAGE being acknowledged e.g.:
+    #
+    #     connection.ack(message.headers['ack'])
+    #
+    # In summary, the behavior is protocol level dependent, see the specifications
+    # and comments in the code.
+    def ack(message_or_ack_id, headers = {})
       raise Stomp::Error::NoCurrentConnection if @closed_check && closed?
       raise Stomp::Error::ProtocolErrorEmptyHeaderKey if headers.has_key?("")
       raise Stomp::Error::ProtocolErrorEmptyHeaderValue if @protocol == Stomp::SPL_10 && headers.has_value?("")
-      raise Stomp::Error::MessageIDRequiredError if message_id.nil? || message_id == ""
+      raise Stomp::Error::MessageIDRequiredError if message_or_ack_id.nil? || message_or_ack_id == ""
       headers = headers.symbolize_keys
 
       case @protocol
         when Stomp::SPL_12
-          # The ACK frame MUST include an id header matching the ack header 
+          # The ACK frame MUST include an "id" header matching the "ack" header 
           # of the MESSAGE being acknowledged.
-          headers[:id] = message_id
+          headers[:id] = message_or_ack_id
         when Stomp::SPL_11
-          # ACK has two REQUIRED headers: message-id, which MUST contain a value 
-          # matching the message-id for the MESSAGE being acknowledged and 
-          # subscription, which MUST be set to match the value of the subscription's 
+          # ACK has two REQUIRED headers: "message-id", which MUST contain a value 
+          # matching the message-id header of the MESSAGE being acknowledged and 
+          # "subscription", which MUST be set to match the value of SUBSCRIBE's 
           # id header.
-          headers[:'message-id'] = message_id
+          headers[:'message-id'] = message_or_ack_id
           raise Stomp::Error::SubscriptionRequiredError unless headers[:subscription]
         else # Stomp::SPL_10
-          # ACK has one required header, message-id, which must contain a value 
+          # ACK has one required header, "message-id", which must contain a value 
           # matching the message-id for the MESSAGE being acknowledged.
-          headers[:'message-id'] = message_id
+          headers[:'message-id'] = message_or_ack_id
       end
       _headerCheck(headers)
       slog(:on_ack, log_params, headers)
@@ -235,23 +278,31 @@ module Stomp
     end
 
     # STOMP 1.1+ NACK.
-    def nack(message_id, headers = {})
+    #
+    # When the connection protocol level is 1.1  the message_or_ack_id parameter
+    # should match the message-id header of the MESSAGE being acknowledged.
+    #
+    # When the connection protocol level is 1.2 the message_or_ack_id parameter
+    # should match the ack header of the MESSAGE being acknowledged.
+    #
+    # Behavior is protocol level dependent, see the specifications and comments below.
+    def nack(message_or_ack_id, headers = {})
       raise Stomp::Error::NoCurrentConnection if @closed_check && closed?
       raise Stomp::Error::UnsupportedProtocolError if @protocol == Stomp::SPL_10
       raise Stomp::Error::ProtocolErrorEmptyHeaderKey if headers.has_key?("")
-      raise Stomp::Error::MessageIDRequiredError if message_id.nil? || message_id == ""
+      raise Stomp::Error::MessageIDRequiredError if message_or_ack_id.nil? || message_or_ack_id == ""
       headers = headers.symbolize_keys
       case @protocol
         when Stomp::SPL_12
           # The NACK frame MUST include an id header matching the ack header 
           # of the MESSAGE being acknowledged.
-          headers[:id] = message_id
+          headers[:id] = message_or_ack_id
         else # Stomp::SPL_11 only
           # NACK has two REQUIRED headers: message-id, which MUST contain a value 
           # matching the message-id for the MESSAGE being acknowledged and 
           # subscription, which MUST be set to match the value of the subscription's 
           # id header.
-          headers[:'message-id'] = message_id
+          headers[:'message-id'] = message_or_ack_id
           raise Stomp::Error::SubscriptionRequiredError unless headers[:subscription]
       end
       _headerCheck(headers)
@@ -285,12 +336,13 @@ module Stomp
 
     # Subscribe subscribes to a destination.  A subscription name is required.
     # For Stomp 1.1+ a session unique subscription ID is also required.
-    def subscribe(name, headers = {}, subId = nil)
+    def subscribe(destination, headers = {}, subId = nil)
       raise Stomp::Error::NoCurrentConnection if @closed_check && closed?
       raise Stomp::Error::ProtocolErrorEmptyHeaderKey if headers.has_key?("")
       raise Stomp::Error::ProtocolErrorEmptyHeaderValue if @protocol == Stomp::SPL_10 && headers.has_value?("")
       headers = headers.symbolize_keys
-      headers[:destination] = name
+      raise Stomp::Error::DestinationRequired unless destination
+      headers[:destination] = destination
       if @protocol >= Stomp::SPL_11
         raise Stomp::Error::SubscriptionRequiredError if (headers[:id].nil? && subId.nil?)
         headers[:id] = subId if headers[:id].nil?
@@ -300,7 +352,7 @@ module Stomp
 
       # Store the subscription so that we can replay if we reconnect.
       if @reliable
-        subId = name if subId.nil?
+        subId = destination if subId.nil?
         raise Stomp::Error::DuplicateSubscription if @subscriptions[subId]
         @subscriptions[subId] = headers
       end
@@ -310,12 +362,13 @@ module Stomp
 
     # Unsubscribe from a destination.   A subscription name is required.
     # For Stomp 1.1+ a session unique subscription ID is also required.
-    def unsubscribe(dest, headers = {}, subId = nil)
+    def unsubscribe(destination, headers = {}, subId = nil)
       raise Stomp::Error::NoCurrentConnection if @closed_check && closed?
       raise Stomp::Error::ProtocolErrorEmptyHeaderKey if headers.has_key?("")
       raise Stomp::Error::ProtocolErrorEmptyHeaderValue if @protocol == Stomp::SPL_10 && headers.has_value?("")
       headers = headers.symbolize_keys
-      headers[:destination] = dest
+      raise Stomp::Error::DestinationRequired unless destination
+      headers[:destination] = destination
       if @protocol >= Stomp::SPL_11
         raise Stomp::Error::SubscriptionRequiredError if (headers[:id].nil? && subId.nil?)
         headers[:id] = subId unless headers[:id]
@@ -324,7 +377,7 @@ module Stomp
       slog(:on_unsubscribe, log_params, headers)
       transmit(Stomp::CMD_UNSUBSCRIBE, headers)
       if @reliable
-        subId = dest if subId.nil?
+        subId = destination if subId.nil?
         @subscriptions.delete(subId)
       end
     end
@@ -337,6 +390,7 @@ module Stomp
       raise Stomp::Error::ProtocolErrorEmptyHeaderKey if headers.has_key?("")
       raise Stomp::Error::ProtocolErrorEmptyHeaderValue if @protocol == Stomp::SPL_10 && headers.has_value?("")
       headers = headers.symbolize_keys
+      raise Stomp::Error::DestinationRequired unless destination
       headers[:destination] = destination
       _headerCheck(headers)
       slog(:on_publish, log_params, message, headers)
@@ -427,8 +481,9 @@ module Stomp
       super_result = __old_receive()
       if super_result.nil? && @reliable && !closed?
         errstr = "connection.receive returning EOF as nil - resetting connection.\n"
-        slog(:on_miscerr, log_params, "es_recv: " + errstr)
-        $stderr.print errstr
+        unless slog(:on_miscerr, log_params, "es_recv: " + errstr)
+          $stderr.print errstr
+        end
 
         # !!! This initiates a re-connect !!!
         # The call to __old_receive() will in turn call socket().  Before
