@@ -26,10 +26,10 @@ module Stomp
   SPL_11 = "1.1"
   SPL_12 = "1.2"
 
-  # Stomp 1.0 and 1.1
+  # Stomp 1.0 and 1.1 and 1.2
   SUPPORTED = [SPL_10, SPL_11, SPL_12]
 
-  # 1.9 Encoding Name
+  # UTF-8 Encoding Name
   UTF8 = "UTF-8"
   #
   # Octet 0
@@ -175,6 +175,10 @@ module Stomp
     ["SRP-RSA-AES-256-CBC-SHA","TLSv1/SSLv3",256,256],
   ]
 
+  HAND_SHAKE_DATA = "\x15\x03\x03\x00\x02\x02\n"
+
+  original_verbose, $VERBOSE = $VERBOSE, nil # try to shut off warnings
+
   # stomp URL regex pattern, for e.g. login:passcode@host:port or host:port
   URL_REPAT = '((([\w~!@#$%^&*()\-+=.?:<>,.]*\w):([\w~!@#$%^&*()\-+=.?:<>,.]*))?@)?([\w\.\-]+):(\d+)'
 
@@ -182,4 +186,6 @@ module Stomp
   #failover:(stomp+ssl://login1:passcode1@remotehost1:61612,stomp://login2:passcode2@remotehost2:61613)
   FAILOVER_REGEX = /^failover:(\/\/)?\(stomp(\+ssl)?:\/\/#{URL_REPAT}(,stomp(\+ssl)?:\/\/#{URL_REPAT})*\)(\?(.*))?$/
 
+  $VERBOSE = original_verbose
+
 end # Module Stomp
