@@ -33,7 +33,9 @@ class TestConnection < Test::Unit::TestCase
 
   # Test asynchronous polling.
   def test_poll_async
-    @conn.subscribe("/queue/do.not.put.messages.on.this.queue", :id => "a.no.messages.queue")
+	  sq = ENV['STOMP_ARTEMIS'] ? "jms.queue.queue.do.not.put.messages.on.this.queue" :
+		  "/queue/do.not.put.messages.on.this.queue"
+    @conn.subscribe(sq, :id => "a.no.messages.queue")
     # If the test 'hangs' here, Connection#poll is broken.
     m = @conn.poll
     assert m.nil?
@@ -74,11 +76,9 @@ class TestConnection < Test::Unit::TestCase
   # Test asking for a receipt on disconnect.
   def test_disconnect_receipt
     @conn.disconnect :receipt => "abc123"
-    assert_nothing_raised {
-      assert_not_nil(@conn.disconnect_receipt, "should have a receipt")
-      assert_equal(@conn.disconnect_receipt.headers['receipt-id'],
-        "abc123", "receipt sent and received should match")
-    }
+    assert_not_nil(@conn.disconnect_receipt, "should have a receipt")
+    assert_equal(@conn.disconnect_receipt.headers['receipt-id'],
+      "abc123", "receipt sent and received should match")
   end
 
   # Test ACKs for Stomp 1.0
@@ -91,11 +91,9 @@ class TestConnection < Test::Unit::TestCase
     @conn.subscribe queue, :ack => :client
     @conn.publish queue, "test_stomp#test_client_ack_with_symbol_10"
     msg = @conn.receive
-    assert_nothing_raised {
-      # ACK has one required header, message-id, which must contain a value 
-      # matching the message-id for the MESSAGE being acknowledged.
-      @conn.ack msg.headers['message-id']
-    }
+    # ACK has one required header, message-id, which must contain a value 
+    # matching the message-id for the MESSAGE being acknowledged.
+    @conn.ack msg.headers['message-id']
     checkEmsg(@conn)
   end
 
@@ -110,13 +108,11 @@ class TestConnection < Test::Unit::TestCase
     @conn.subscribe queue, :ack => :client, :id => sid
     @conn.publish queue, "test_stomp#test_client_ack_with_symbol_11"
     msg = @conn.receive
-    assert_nothing_raised {
-      # ACK has two REQUIRED headers: message-id, which MUST contain a value 
-      # matching the message-id for the MESSAGE being acknowledged and 
-      # subscription, which MUST be set to match the value of the subscription's 
-      # id header.
-      @conn.ack msg.headers['message-id'], :subscription => msg.headers['subscription']
-    }
+    # ACK has two REQUIRED headers: message-id, which MUST contain a value 
+    # matching the message-id for the MESSAGE being acknowledged and 
+    # subscription, which MUST be set to match the value of the subscription's 
+    # id header.
+    @conn.ack msg.headers['message-id'], :subscription => msg.headers['subscription']
     checkEmsg(@conn)
   end
 
@@ -131,11 +127,9 @@ class TestConnection < Test::Unit::TestCase
     @conn.subscribe queue, :ack => :client, :id => sid
     @conn.publish queue, "test_stomp#test_client_ack_with_symbol_11"
     msg = @conn.receive
-    assert_nothing_raised {
-      # The ACK frame MUST include an id header matching the ack header 
-      # of the MESSAGE being acknowledged.
-      @conn.ack msg.headers['ack']
-    }
+    # The ACK frame MUST include an id header matching the ack header 
+    # of the MESSAGE being acknowledged.
+    @conn.ack msg.headers['ack']
     checkEmsg(@conn)
   end
 
@@ -203,11 +197,11 @@ class TestConnection < Test::Unit::TestCase
     end
     #
     assert_raise Stomp::Error::NoCurrentConnection do
-      m = @conn.receive
+      _ = @conn.receive
     end
     #
     assert_raise Stomp::Error::NoCurrentConnection do
-      m = @conn.poll
+      _ = @conn.poll
     end
   end
 
@@ -225,13 +219,13 @@ class TestConnection < Test::Unit::TestCase
     conn_subscribe make_destination
     @conn.publish make_destination, "a\0"
     msg = @conn.receive
-    assert_match /^<Stomp::Message headers=/ , msg.to_s
+    assert_match(/^<Stomp::Message headers=/ , msg.to_s)
     checkEmsg(@conn)
   end
   
   # Test that a connection frame is present.
   def test_connection_frame
-  	assert_not_nil @conn.connection_frame
+    assert_not_nil @conn.connection_frame
   end
   
   # Test messages with multiple line ends.
@@ -309,7 +303,7 @@ class TestConnection < Test::Unit::TestCase
     1.upto(@max_threads) do |tnum|
       Thread.new(@conn) do |amq|
         while true
-          received = amq.receive
+          _ = amq.receive
           lock.synchronize do
             msg_ctr += 1
           end
@@ -387,9 +381,7 @@ class TestConnection < Test::Unit::TestCase
   # Test using a nil body.
   def test_nil_body
     dest = make_destination
-    assert_nothing_raised {
-      @conn.publish dest, nil
-    }
+    @conn.publish dest, nil
     conn_subscribe dest
     msg = @conn.receive
     assert_equal "", msg.body
@@ -412,7 +404,7 @@ class TestConnection < Test::Unit::TestCase
     msg = @conn.receive
     assert_equal "txn message", msg.body
     checkEmsg(@conn)
-  end
+  end unless ENV['STOMP_ARTEMIS'] # See Artemis docs for 1.3, page 222
 
   # Test duplicate subscriptions.
   def test_duplicate_subscription
@@ -431,9 +423,7 @@ class TestConnection < Test::Unit::TestCase
   def test_nil_connparms
     @conn.disconnect
     #
-    assert_nothing_raised do
-      @conn = Stomp::Connection.open(user, passcode, host, port, false, 5, nil)
-    end
+    @conn = Stomp::Connection.open(user, passcode, host, port, false, 5, nil)
     checkEmsg(@conn)
   end
 
@@ -454,17 +444,13 @@ class TestConnection < Test::Unit::TestCase
       assert_equal smsg, msg.body
       case @conn.protocol
         when Stomp::SPL_12
-          assert_nothing_raised {
-            @conn.nack msg.headers["ack"]
-            sleep 0.05 # Give racy brokers a chance to handle the last nack before unsubscribe
-            @conn.unsubscribe dest, :id => sid
-          }
+          @conn.nack msg.headers["ack"]
+          sleep 0.05 # Give racy brokers a chance to handle the last nack before unsubscribe
+          @conn.unsubscribe dest, :id => sid
         else # Stomp::SPL_11
-          assert_nothing_raised {
-            @conn.nack msg.headers["message-id"], :subscription => sid
-            sleep 0.05 # Give racy brokers a chance to handle the last nack before unsubscribe
-            @conn.unsubscribe dest, :id => sid
-          }
+          @conn.nack msg.headers["message-id"], :subscription => sid
+          sleep 0.05 # Give racy brokers a chance to handle the last nack before unsubscribe
+          @conn.unsubscribe dest, :id => sid
       end
 
       # phase 2
@@ -476,7 +462,7 @@ class TestConnection < Test::Unit::TestCase
       assert_equal smsg, msg2.body
       checkEmsg(@conn)
     end
-  end unless ENV['STOMP_AMQ11'] # AMQ sends NACK'd messages to a DLQ
+  end unless (ENV['STOMP_AMQ11'] || ENV['STOMP_ARTEMIS'])
 
   # Test to illustrate Issue #44.  Prior to a fix for #44, these tests would
   # fail only when connecting to a pure STOMP 1.0 server that does not 
@@ -492,9 +478,7 @@ class TestConnection < Test::Unit::TestCase
       :reliable => false,
     }
     c = nil
-    assert_nothing_raised {
-      c = Stomp::Connection.new(hash)
-    }
+    c = Stomp::Connection.new(hash)
     c.disconnect if c
     #
     hash = { :hosts => [ 
@@ -504,9 +488,7 @@ class TestConnection < Test::Unit::TestCase
       :reliable => false,
     }
     c = nil
-    assert_nothing_raised {
-      c = Stomp::Connection.new(hash)
-    }
+    c = Stomp::Connection.new(hash)
     c.disconnect if c
   end
 
@@ -519,5 +501,123 @@ class TestConnection < Test::Unit::TestCase
     end
   end
 
+  # Test that methods detect an empty header key.
+  def test_empty_header_key
+    #
+    bad_headers = {"a" => "11", "" => "emptykey", :c => "ccc"}
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.ack("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.nack("dummy_data", bad_headers)
+    end if @conn.protocol != Stomp::SPL_10
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.begin("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.commit("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.abort("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.subscribe("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.unsubscribe("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.publish("dummy_data","dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderKey do
+      @conn.disconnect(bad_headers)
+    end
+  end
+
+  # Test that methods detect an empty header value.
+  # STOMP 1.0 only.
+  def test_empty_header_value
+    if @conn.protocol != Stomp::SPL_10
+      assert true
+      return
+    end
+    #
+    bad_headers = {"a" => "11", "hdra" => "", :c => "ccc"}
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.ack("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.nack("dummy_data", bad_headers)
+    end if @conn.protocol != Stomp::SPL_10
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.begin("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.commit("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.abort("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.subscribe("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.unsubscribe("dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.publish("dummy_data","dummy_data", bad_headers)
+    end
+    #
+    assert_raise Stomp::Error::ProtocolErrorEmptyHeaderValue do
+      @conn.disconnect(bad_headers)
+    end
+  end
+
+  # test issue99, OK values
+  def test_con_iss99_ok
+    return unless host() == "localhost" && port() == 61613
+    #
+    ok_vals = dflt_data_ok()
+    ok_vals.each do |hsv|
+      conn = Stomp::Connection.new(hsv)
+      conn.disconnect
+    end
+  end
+
+  def test_conn_nodest_sub
+    assert_raise Stomp::Error::DestinationRequired do
+      @conn.subscribe(nil)
+    end
+  end
+
+  def test_conn_nodest_unsub
+    assert_raise Stomp::Error::DestinationRequired do
+      @conn.unsubscribe(nil)
+    end
+  end
+
+  def test_conn_nodest_pub
+    assert_raise Stomp::Error::DestinationRequired do
+      @conn.publish(nil, "msg")
+    end
+  end
+
 end
 
